//////////////////////////////////////////////////////////
//
// AVIDLG.DLL - DLL to play avi video in a dialog box
//
// Programmer: Kent Reisdorph, Compuserve ID 75522,1174
// Version: .95
// Date: May 17, 1995
//
// Usage: AviDlg(hWnd, "Program Name", "Version", "Date", "filename.avi");
//
//////////////////////////////////////////////////////////

#include <string.h>
#include <stdio.h>
#include <mmsystem.h>
#include <digitalv.h>
#include <windowsx.h>
#pragma hdrstop

#include "avidlg.h"
#include "avidlg.rh"

HINSTANCE hMod;
DLGPROC dlgProc;
const char far* Title;
const char far* Version;
const char far* Date;
const char far* Filename;
char Caption[50];
static BOOL DeviceOpen;
static UINT DeviceID;
int RightMargin;
int BottomMargin;

// DLL entry point
int FAR PASCAL LibMain (HINSTANCE hInst, WORD, WORD, LPSTR)
{
	hMod = hInst;
	return TRUE;
}

// DLL exit point - no cleanup necessary in this case
// but it's here in case you need it.
int FAR PASCAL WEP (int /*nParameter*/)
{
	return TRUE;
}

// our entry function
void _export AviDlg(HWND parent,
	const char far* title,
	const char far* version,
	const char far* date,
	const char far* filename)
{
// copy passed strings to our global variables
	Title = title;
	Version = version;
	Date = date;
	Filename = filename;
	RightMargin = GetSystemMetrics(SM_CYCAPTION) * 1.5;
	BottomMargin = RightMargin * 3;
	sprintf(Caption, "About %s", title);
// setup dialog procedure
	dlgProc = (DLGPROC)MakeProcInstance((FARPROC)About, hMod);
// jump in
	CreateDialog(hMod, "AboutBox", parent, dlgProc);
 // free dialog procedure
	FreeProcInstance((FARPROC) dlgProc);
}

// dialog procedure
BOOL CALLBACK About(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg) {
// startup stuff
		case WM_INITDIALOG : {
			HWND hwndCtl;
// add text to static controls
			hwndCtl = GetDlgItem(hwndDlg, IDC_TITLE);
			SetWindowText(hwndCtl, Title);
			hwndCtl = GetDlgItem(hwndDlg, IDC_VERSION);
			SetWindowText(hwndCtl, Version);
			hwndCtl = GetDlgItem(hwndDlg, IDC_DATE);
			SetWindowText(hwndCtl, Date);
// set dialog box caption
			SetWindowText(hwndDlg, Caption);
// open avi device
			DWORD ErrCode = OpenAvi(hwndDlg);
			BOOL Bad = CheckError(ErrCode);
// size the static to the size of the video image
// size the dialog based on the size of the video image and left
// and right margins. move the OK button to the center of the dialog
			RECT sRect;
			RECT dlgRect;
			RECT vRect = GetVideoRect();
			int vWidth = vRect.right - vRect.left;
			int vHeight = vRect.bottom - vRect.top;
			hwndCtl = GetDlgItem(hwndDlg, IDC_AVIWINDOW);
			GetWindowRect(hwndCtl, &sRect);
			GetWindowRect(hwndDlg, &dlgRect);
			POINT point;
			point.x = sRect.left;
			point.y = sRect.top;
			ScreenToClient(hwndDlg, &point);
			dlgRect.right = point.x + vWidth + RightMargin;
			dlgRect.bottom = point.y + vHeight + BottomMargin;
			MoveWindow(hwndDlg, dlgRect.left, dlgRect.top,
									dlgRect.right, dlgRect.bottom, FALSE);
			MoveWindow(hwndCtl, point.x, point.y, vWidth, vHeight, FALSE);
			hwndCtl = GetDlgItem(hwndDlg, IDOK);
			GetWindowRect(hwndCtl, &sRect);
			GetWindowRect(hwndDlg, &dlgRect);
			int bWidth = sRect.right - sRect.left;
			int bHeight = sRect.bottom - sRect.top;
			point.x = (((dlgRect.right - dlgRect.left) / 2) + dlgRect.left)
									- (bWidth/2);
			point.y = (dlgRect.bottom) - (bHeight + 20);
			ScreenToClient(hwndDlg, &point);
			MoveWindow(hwndCtl, point.x, point.y, bWidth, bHeight, FALSE);
// play the avi
			if (!Bad) ErrCode = PlayAvi(hwndDlg);
// report if error - probably only for debugging
			CheckError(ErrCode);
			return(TRUE);
		}
		case WM_COMMAND :
// if OK clicked then close avi device and exit
			if ( GET_WM_COMMAND_ID(wParam, lParam) == IDOK ) {
				mciSendCommand(DeviceID, MCI_CLOSE, 0, NULL);
				DeviceOpen = FALSE;
				EndDialog(hwndDlg, TRUE);
				return(TRUE);
			}
			break;
// This message tells us that the video has reached the end
// so we rewind and start playback again.
// This is not strictly necessary as digitalv.h provides a flag
// for continuous play. But this method demonstrates the use of
// the MCI_NOTIFY flag and MM_MCINOTIFY message.
		case MM_MCINOTIFY :
			RewindAvi();
			PlayAvi(hwndDlg);
			return(TRUE);
		case WM_SYSCOMMAND :
			if (wParam == SC_CLOSE) {
				mciSendCommand(DeviceID, MCI_CLOSE, 0, NULL);
				DeviceOpen = FALSE;
				EndDialog(hwndDlg, TRUE);
				return(TRUE);
			}
			break;
	}
// default processing
	return(FALSE);
}

// opens the avi device
DWORD OpenAvi(HWND hwndDlg)
{
	UINT devID;
	DWORD flags = MCI_DGV_OPEN_PARENT | MCI_DGV_OPEN_WS |
		MCI_OPEN_ELEMENT | MCI_OPEN_TYPE;
	MCI_DGV_OPEN_PARMS Open;
	Open.lpstrDeviceType = "avivideo";
	Open.lpstrElementName = (LPSTR)Filename;
// you can try different styles here but this works best
	Open.dwStyle = WS_CHILD | WS_VISIBLE;
// set the parent window to the static we set up for it
	Open.hWndParent = GetDlgItem(hwndDlg, IDC_AVIWINDOW);
	if (DeviceOpen) {
		devID = DeviceID;
	}
	else devID = 0;
// open device
	DWORD ErrCode = mciSendCommand(devID,
		MCI_OPEN, flags, (DWORD)(LPVOID) &Open);
// no error - set global flags
	if (!ErrCode) {
		DeviceOpen = TRUE;
		DeviceID = Open.wDeviceID;
	}
	return ErrCode;
}

// play video
DWORD PlayAvi(HWND hwndDlg)
{
// tells dialog procedure when video reaches end
	DWORD flags = MCI_NOTIFY;
	MCI_PLAY_PARMS Play;
// callback is set to dialog hwnd so our dialog procedure
// gets the message
	Play.dwCallback = MAKELONG(hwndDlg,0);
// play
	return mciSendCommand(DeviceID,
		MCI_PLAY, flags, (DWORD)(LPVOID) &Play);
}

// rewind video
void RewindAvi()
{
	MCI_SEEK_PARMS Seek;
	mciSendCommand(DeviceID, MCI_SEEK,
		MCI_SEEK_TO_START, (DWORD)(LPVOID) &Seek);
}

RECT GetVideoRect()
{
	MCI_DGV_WHERE_PARMS Parms;
	DWORD flags = MCI_WAIT | MCI_DGV_WHERE_SOURCE;
	mciSendCommand(DeviceID,
		MCI_WHERE, flags, (DWORD)(LPVOID) &Parms);
	return Parms.rc;
}

BOOL CheckError(DWORD errCode)
{
	switch (errCode) {
		case MCIERR_DEVICE_NOT_INSTALLED :
		case MCIERR_CANNOT_LOAD_DRIVER :
		case MCIERR_FILE_NOT_FOUND :
		case MCIERR_OUT_OF_MEMORY :
		if (DeviceOpen) mciSendCommand(DeviceID, MCI_CLOSE, 0, NULL);
		DeviceOpen = FALSE;
// the following three statements are for diagnostics. use them for
// debugging but you may want to take them out of the final product
//		char ErrMsg[256];
//		mciGetErrorString(ErrCode, ErrMsg, sizeof(ErrMsg));
//		MessageBox(0, ErrMsg, "Error", MB_OK);
		return TRUE;
	}
	return FALSE;
}

